@@ -1,7 +1,9 @@
 #ifndef LIBRAPID_INCLUDE
 #define LIBRAPID_INCLUDE
 
+#include <librapid/utils/utils.hpp>
 #include <librapid/math/rapid_math.hpp>
 #include <librapid/ndarray/ndarray.hpp>
+#include <librapid/network/network.hpp>
 
 #endif // LIBRAPID_INCLUDE
\ No newline at end of file
