@@ -1,6 +1,7 @@
 #ifndef LIBRAPID_CBLAS_API
 #define LIBRAPID_CBLAS_API
 
+#include <thread>
 #include <librapid/config.hpp>
 
 #ifdef LIBRAPID_CBLAS
@@ -40,10 +41,10 @@ namespace librapid
 	#endif // LIBRAPID_CBLAS
 
 		template<typename T>
-		LR_INLINE void cblas_gemv(char order, bool trans, lr_int m, lr_int n,
-								  T alpha, T *__restrict a, lr_int lda,
-								  T *__restrict x, lr_int incx, double beta,
-								  T *__restrict y, lr_int incy)
+		LR_INLINE void cblas_gemv_no_blas(char order, bool trans, lr_int m, lr_int n,
+										  T alpha, T *__restrict a, lr_int lda,
+										  T *__restrict x, lr_int incx, T beta,
+										  T *__restrict y, lr_int incy)
 		{
 			lr_int _lda = trans ? 1 : lda;
 			lr_int _fda = trans ? lda : 1;
@@ -73,11 +74,19 @@ namespace librapid
 			}
 		}
 
+		template<typename T>
+		LR_INLINE void cblas_gemv(char order, bool trans, lr_int m, lr_int n, T alpha,
+								  T *__restrict a, lr_int lda, T *__restrict x, lr_int incx,
+								  T beta, T *__restrict y, lr_int incy)
+		{
+			cblas_gemv_no_blas(order, trans, m, n, alpha, a, lda, x, incx, beta, y, incy);
+		}
+
 	#ifdef LIBRAPID_CBLAS
 		template<>
 		LR_INLINE void cblas_gemv(char order, bool trans, lr_int m, lr_int n,
 								  float alpha, float *__restrict a, lr_int lda,
-								  float *__restrict x, lr_int incx, double beta,
+								  float *__restrict x, lr_int incx, float beta,
 								  float *__restrict y, lr_int incy)
 		{
 			CBLAS_ORDER blas_order{};
@@ -118,74 +127,78 @@ namespace librapid
 	#endif // LIBRAPID_CBLAS
 
 		template<typename T>
-		LR_INLINE void cblas_gemm(char order, bool trans_a, bool trans_b, lr_int m,
-								  lr_int n, lr_int k, T alpha,
-								  T *__restrict a, lr_int lda, T *__restrict b,
-								  lr_int ldb, T beta, T *__restrict c, lr_int ldc)
+		LR_INLINE void cblas_gemm_no_blas(char order, bool trans_a, bool trans_b, lr_int m,
+										  lr_int n, lr_int k, T alpha,
+										  T *__restrict a, lr_int lda, T *__restrict b,
+										  lr_int ldb, T beta, T *__restrict c, lr_int ldc)
 		{
+			lr_int outer, inner, sub;
+			lr_int temp_a, index_c;
+
 			lr_int _lda = trans_a ? 1 : lda;
 			lr_int _fda = trans_a ? lda : 1;
 
-			lr_int _ldb = trans_a ? 1 : ldb;
-			lr_int _fdb = trans_a ? ldb : 1;
+			lr_int _ldb = trans_b ? 1 : ldb;
+			lr_int _fdb = trans_b ? ldb : 1;
 
-			lr_int index_a, index_b, index_c;
+			lr_int _ldc = trans_b ? 1 : ldc;
+			lr_int _fdc = trans_b ? ldc : 1;
 
-			// Only run in parallel if arrays are smaller than a
-			// given size.Running in parallel on smaller matrices
-			// will result in slower code.Note, a branch is used
-			// in preference to #pragma omp ... if (...) because
-			// that requires runtime evaluation of a condition to
-			// set up threads, which adds a significant overhead
-			if (m * n * k < 25000)
+			if (m * n * k < 2500)
 			{
-				for (lr_int outer = 0; outer < m; ++outer)
+				for (outer = 0; outer < m; ++outer)
 				{
-					for (lr_int inner = 0; inner < n; ++inner)
+					for (inner = 0; inner < n; ++inner)
 					{
-						index_c = outer * ldc + inner;
+						temp_a = outer * _lda;
+						index_c = inner * _fdc + outer * _ldc;
 
-						if (beta != 0)
-							c[index_c] += c[index_c] * beta;
-						else
+						if (beta == 0)
 							c[index_c] = 0;
+						else
+							c[index_c] += c[index_c] * beta;
 
-						for (lr_int sub = 0; sub < k; sub++)
+						for (sub = 0; sub < k; ++sub)
 						{
-							index_a = outer * _lda + sub * _fda;
-							index_b = inner * _ldb + sub * _fdb;
-
-							c[index_c] += a[index_a] * b[index_b];
+							c[index_c] += a[temp_a + sub * _fda] * b[sub * _ldb + inner * _fdb];
 						}
 					}
 				}
 			}
 			else
 			{
-			#pragma omp parallel for shared(a, b, c, m, n, k, _lda, _ldb, ldc, _fda, _fdb, beta) private(index_a, index_b, index_c) default(none)
-				for (lr_int outer = 0; outer < m; ++outer)
+			#pragma omp parallel for shared(m, n, k, _lda, _fda, _ldb, _fdb, _ldc, _fdc, alpha, beta, a, b, c) private(outer, inner, sub, temp_a, index_c) default(none)
+				for (outer = 0; outer < m; ++outer)
 				{
-					for (lr_int inner = 0; inner < n; ++inner)
+					for (inner = 0; inner < n; ++inner)
 					{
-						index_c = outer * ldc + inner;
+						temp_a = outer * _lda;
+						index_c = inner * _fdc + outer * _ldc;
 
-						if (beta != 0)
-							c[index_c] += c[index_c] * beta;
-						else
+						if (beta == 0)
 							c[index_c] = 0;
+						else
+							c[index_c] += c[index_c] * beta;
 
-						for (lr_int sub = 0; sub < k; sub++)
+						for (sub = 0; sub < k; ++sub)
 						{
-							index_a = outer * _lda + sub * _fda;
-							index_b = inner * _ldb + sub * _fdb;
-
-							c[index_c] += a[index_a] * b[index_b];
+							c[index_c] += a[temp_a + sub * _fda] * b[sub * _ldb + inner * _fdb];
 						}
 					}
 				}
 			}
 		}
 
+		template<typename T>
+		LR_INLINE void cblas_gemm(char order, bool trans_a, bool trans_b, lr_int m,
+								  lr_int n, lr_int k, T alpha,
+								  T *__restrict a, lr_int lda, T *__restrict b,
+								  lr_int ldb, T beta, T *__restrict c, lr_int ldc)
+		{
+			cblas_gemm_no_blas(order, trans_a, trans_b, m, n, k,
+							   alpha, a, lda, b, ldb, beta, c, ldc);
+		}
+
 	#ifdef LIBRAPID_CBLAS
 		template<>
 		LR_INLINE void cblas_gemm(char order, bool trans_a, bool trans_b, lr_int m,
